(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.spjaeld = factory());
}(this, (function () { 'use strict';

var callAndSetInterval = function callAndSetInterval(fn, delay) {
  fn();
  return setInterval(fn, delay);
};

var processQueue = function processQueue(fun, _ref) {
  var context = _ref.context,
      parameters = _ref.parameters,
      resolver = _ref.resolver,
      rejecter = _ref.rejecter;

  try {
    resolver(fun.apply(context, parameters));
  } catch (err) {
    rejecter(err);
  }
};

var rateLimitFn = function rateLimitFn(fun) {
  var delay = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 300;

  var queue = [];
  var timer = null;

  return function rateLimitedFn() {
    for (var _len = arguments.length, parameters = Array(_len), _key = 0; _key < _len; _key++) {
      parameters[_key] = arguments[_key];
    }

    var resolver = void 0;
    var rejecter = void 0;
    var invocationPromise = new Promise(function (resolve, reject) {
      resolver = resolve;
      rejecter = reject;
    });
    queue.push({
      context: this, parameters: parameters, resolver: resolver, rejecter: rejecter
    });
    if (!timer) {
      timer = callAndSetInterval(function () {
        if (queue.length) {
          processQueue(fun, queue.shift());
        } else {
          clearInterval(timer);
          timer = null;
        }
      }, delay);
    }
    return invocationPromise;
  };
};

return rateLimitFn;

})));
